/******************************************************************************/
/* DSP_LOAD.C								      */
/******************************************************************************/

#include "st_def.h"

/* -------------------------------------------------------------------------- */
/* SOUND trap calling routine */

extern	long	SOUND(int, ...);

#define	locksnd()				SOUND(0x80)
#define	unlocksnd()				SOUND(0x81)
#define	soundcmd(a,b)				SOUND(0x82,a,b)
#define	setbuffer(a,b,c)			SOUND(0x83,a,b,c)
#define	setmode(a)				SOUND(0x84,a)
#define	settrack(a,b)				SOUND(0x85,a,b)
#define	setmontrack(a)				SOUND(0x86,a)
#define	setinterrupt(a,b)			SOUND(0x87,a,b)
#define	buffoper(a)				SOUND(0x88,a)
#define	dsptristate(a,b)			SOUND(0x89,a,b)
#define	gpio(a,b)				SOUND(0x8A,a,b)
#define	devconnect(a,b,c,d,e)			SOUND(0x8B,a,b,c,d,e)
#define	sndstatus(a)				SOUND(0x8C,a)
#define	buffptr(a)				SOUND(0x8D,a)

/* DSP XBIOS functions (trap #14) */

extern	long	xbios(int, ...);

#define	Dsp_DoBlock(a,b,c,d)			xbios(96,a,b,c,d)
#define	Dsp_BlkHandShake(a,b,c,d)		xbios(97,a,b,c,d)
#define	Dsp_BlkUnpacked(a,b,c,d)		xbios(98,a,b,c,d)
#define	Dsp_InStream(a,b,c,d)			xbios(99,a,b,c,d)
#define	Dsp_OutStream(a,b,c,d)			xbios(100,a,b,c,d)
#define	Dsp_IOStream(a,b,c,d,e,f)		xbios(101,a,b,c,d,e,f)
#define	Dsp_RemoveInterrupts(a)			xbios(102,a)
#define	Dsp_GetWordSize()			xbios(103)
#define	Dsp_Lock()				xbios(104)
#define	Dsp_Unlock()				xbios(105)
#define	Dsp_Available(a,b)			xbios(106,a,b)
#define	Dsp_Reserve(a,b)			xbios(107,a,b)
#define	Dsp_LoadProg(a,b,c)			xbios(108,a,b,c)
#define	Dsp_ExecProg(a,b,c)			xbios(109,a,b,c)
#define	Dsp_ExecBoot(a,b,c)			xbios(110,a,b,c)
#define	Dsp_LodToBinary(a,b)			xbios(111,a,b)
#define	Dsp_TriggerHC(a)			xbios(112,a)
#define	Dsp_RequestUniqueAbility()		xbios(113)
#define	Dsp_GetProgAbility()			xbios(114)
#define	Dsp_FlushSubroutines()			xbios(115)
#define	Dsp_LoadSubroutine(a,b,c)		xbios(116,a,b,c)
#define	Dsp_InqSubrAbility(a)			xbios(117,a)
#define	Dsp_RunSubroutine(a)			xbios(118,a)
#define	Dsp_Hf0(a)				xbios(119,a)
#define	Dsp_Hf1(a)				xbios(120,a)
#define	Dsp_Hf2()				xbios(121)
#define	Dsp_Hf3()				xbios(122)
#define	Dsp_BlkWords(a,b,c,d)			xbios(123,a,b,c,d)
#define	Dsp_BlkBytes(a,b,c,d)			xbios(124,a,b,c,d)
#define	Dsp_HStat()				xbios(125)
#define Dsp_SetVectors(a,b)			xbios(126,a,b)
#define Dsp_MultBlocks(a,b,c,d)			xbios(127,a,b,c,d)

/* -------------------------------------------------------------------------- */

int	dsp_reloadprg( void );
int	dsp_loadprg( void );

/* -------------------------------------------------------------------------- */

extern	unsigned char	gl_pathname[];
extern	unsigned char	gl_sysname[];

/* -------------------------------------------------------------------------- */

static	unsigned char	*dsp_routine = { "I2S.LOD" };
static	int	dsp_loaded = FALSE;

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

int	dsp_reloadprg()
{
	dsp_loaded = FALSE;
	return(dsp_loadprg());
}

/* -----------------------------------------------------------------------------
 */

int	dsp_loadprg()
{
register DMABUFFER *olddta;
register void	*buffer;
register long	ret;
DMABUFFER filedta;
unsigned char	pathname[ MAX_PATHNAME ];

	if (dsp_loaded)
		return(TRUE);

	strcpy(pathname, gl_pathname);
	strcat(pathname, gl_sysname);
	strcat(pathname, dsp_routine);

	olddta = Fgetdta();
	Fsetdta(&filedta);
	ret = Fsfirst(pathname, 0);
	Fsetdta(olddta);
	if (ret < 0)
	{
		st_error_message(E_I2S_NOLODFILE);
		return(FALSE);
	}

	if (locksnd() < 0)
	{
		unlocksnd();
		if (locksnd() < 0)
		{
			st_error_message(E_I2S_SOUNDLOCKED);
			return(FALSE);
		}
	}
	soundcmd(2,80);
	soundcmd(3,80);	
	soundcmd(4,2);	
	setmode(1);
	settrack(0,0);

	buffer = calloc(1, (long)(filedta.d_fsize * 2));
	if (buffer == NULL)
	{
		st_error_message(E_I2S_NOMEMORY);
		unlocksnd();
		return(FALSE);
	}
	ret = Dsp_LoadProg(pathname, 3, buffer);
	Mfree(buffer);
	unlocksnd();

	if (ret < 0)
	{
		st_error_message(E_I2S_LOADERROR);
		return(FALSE);
	}
	dsp_loaded = TRUE;
	return(TRUE);
}

/******************************************************************************/
/* End of DSP_LOAD.C							      */
/******************************************************************************/
