/******************************************************************************/
/* CD_INIT.C								      */
/******************************************************************************/

#include "st_def.h"
#include "st_struc.h"
#include "st_aes.h"
#include "st_bind.h"
#include "cd_bind.h"

/* -------------------------------------------------------------------------- */

extern	CDROM	*gl_actdocu;

extern	GRECT	cd_totlrect;

extern	long	cd_docuwide;
extern	long	cd_docuhite;

extern	long	cd_visx;
extern	long	cd_visy;

/* -------------------------------------------------------------------------- */

CDROM	*undo_gl_actdocu = NULL;

/* -------------------------------------------------------------------------- */

static	long	undo_cd_docuwide;	/* document width on the screen */
static	long	undo_cd_docuhite;	/* document hite on the screen */

static	long	undo_cd_visx;		/* visual x start position of the document */
static	long	undo_cd_visy;		/* visual y start position of the document */

/* -------------------------------------------------------------------------- */

static	int	cd_undo_error( void );

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Clears uncomplete undo buffer and shows an error message whenever a memory
 * allocation error occurs while saving the undo buffer
 */

static	int	cd_undo_error()
{
	cd_undo_free();
	return(st_error_message(E_UNDOERR) - 1);
}

/* -----------------------------------------------------------------------------
 * restores undo status and flushes buffers, resets pointers and initializes
 * counters
 */

void	cd_undo_free()
{
register CDROM	*undodocu = undo_gl_actdocu;

	if (undodocu == NULL)
		return;

	cd_free(undodocu);

	undo_gl_actdocu = NULL;
	cd_setmenu_undo();
	return;
}

/* -----------------------------------------------------------------------------
 */

int	cd_saveundo()
{
register CDROM	*docu = gl_actdocu;
register CDROM	*undodocu;
register ITEM	*newitem;
register ITEM	*item;
register ITEM	*prev;

	st_mouse_busy();
	cd_undo_free();

	undodocu = (CDROM *)calloc((size_t)1, (size_t)sizeof(CDROM));
	if (undodocu == NULL)
	{
		return(cd_undo_error());
	}
	undo_gl_actdocu = undodocu;

	undodocu->Settings = docu->Settings;

	strcpy(undodocu->CdRomName, docu->CdRomName);
	strcpy(undodocu->PathName, docu->PathName);

	undodocu->NoSessions = docu->NoSessions;
	undodocu->NoTracks = docu->NoTracks;
	undodocu->NoItems = docu->NoItems;

	undodocu->NoSelItems = docu->NoSelItems;

	prev = NULL;
	for (item = docu->FirstItem ; item ; item = item->next)
	{
		newitem = (ITEM *)calloc((size_t)1, (size_t)sizeof(ITEM));
		if (newitem == NULL)
		{
			return(cd_undo_error());
		}

		if (prev == NULL)
			undodocu->FirstItem = newitem;
		else	prev->next = newitem;
		newitem->prev = prev;
		undodocu->LastItem = newitem;
		prev = newitem;

		newitem->Settings = item->Settings;
		newitem->Length = item->Length;

		newitem->FrameStart = item->FrameStart;
		newitem->FrameLength = item->FrameLength;

		strcpy(newitem->Comments, item->Comments);
		strcpy(newitem->PathName, item->PathName);

		newitem->Hite = item->Hite;
		newitem->OffsetY = item->OffsetY;
		newitem->StartX = item->StartX;

		if (docu->SelItem == item)
		{
			undodocu->SelItem = newitem;
		}
	}
	undo_cd_docuwide = cd_docuwide;
	undo_cd_docuhite = cd_docuhite;

	undo_cd_visx = cd_visx;
	undo_cd_visy = cd_visy;

	cd_setmenu_undo();
	return(TRUE);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	cd_undo()
{
register CDROM	*undodocu = undo_gl_actdocu;
register CDROM	*docu = gl_actdocu;

	if (undodocu == NULL)
		return;

	st_mouse_busy();

	cd_docuwide = undo_cd_docuwide;
	cd_docuhite = undo_cd_docuhite;

	cd_visx = undo_cd_visx;
	cd_visy = undo_cd_visy;

	gl_actdocu = undodocu;
	undo_gl_actdocu = docu;
	cd_undo_free();

	cd_update_time(gl_actdocu);
	cd_set_maxvisxy();
	cd_redraw(&cd_totlrect);
	st_set_menu();
	return;
}

/******************************************************************************/
/* End of CD_INIT.C							      */
/******************************************************************************/
