;********************************************************************************
;* MINE.S									*
;********************************************************************************

;********************************************************************************
;* INCLUDE FILES								*
;********************************************************************************

	include	"hd_error.h"
	include	"hd_mfp.h"
	include	"hd_sysvr.h"
	include	"hd_macro.h"
	include	"hd_68030.h"
	include	"hd_acsi.h"
	include	"hd_scsi2.h"

;********************************************************************************
;* LOCAL EQUATES								*
;********************************************************************************

RETRIES		equ	$3		; maximum number of retries

;****************************************
;* EXTENDED SCSI READ COMMAND		*
;****************************************

XREAD		equ	$28		; SCSI only
LCMD		equ	10		; extended command length (10 bytes)

;****************************************
;* DIFFERENT TIMEOUT PERIODS		*
;****************************************

scxltmout	equ	12001		; SCSI extra long-timeout (>1 min)
scstmout	equ	101		; SCSI short-timeout (>500 ms)
scltmout	equ	201		; SCSI long-timeout (>1000 ms)
rcaltm		equ	801		; time for drive recalibration (>4s)

;******************************************************************************
;*	CODE SECTION
;******************************************************************************

	CSECT	aaaa,code

;******************************************************************************
;* ROUTINES ACCESSED FROM OTHER MODULES
;******************************************************************************

	XDEF	_Lrwabs

;********************************************************************************
;* READ SECTORS FROM SCSI DRIVE							*
;*										*
;* Synopsis:									*
;*	_Lrwabs(rw, buf, count, recno, dev, lrecno)				*
;*		WORD rw		$4(sp).w	; flags				*
;*		char *buf	$6(sp).l	; buffer address		*
;*		WORD count	$a(sp).w	; number of sectors to read	*
;*		WORD dev	$c(sp).w	; device number			*
;*		LONG lrecno	$e(sp).l	; starting sector number	*
;*										*
;********************************************************************************

;****************************************
;* STACK FRAME OFFSETS			*
;****************************************

saveregs	equ	$8
xrw		equ	saveregs+$4
xbuf		equ	saveregs+$6
xcount		equ	saveregs+$a
xdev		equ	saveregs+$c
xlrecno		equ	saveregs+$e

;****************************************
;* THE SCSI READ ROUTINE		*
;****************************************

_Lrwabs:
read0:	movem.l	d2/a0,-(sp)		; save registers
	st	flock			; lock FIFO

;****************************************
;* Select the SCSI DEVICE		*
;****************************************

sel00:	TMOUT	#scstmout,d1		; set up a short timeout
	move.w	#SPSCSICR,WDL
sel01:	move.w	WDC,d0
	btst	#6,d0			; STILL busy from last time ?
	beq.s	sel10			; if not, it's available
sel02:	CNTDN1	d1,sel01		; else if not timed-out, wait longer
sel03:	bra	sel30			; timed out - read error

sel10:	WSCSI	#$00,#SPSCSITCR		; data out phase
	WSCSI	#$00,#SPSCSIISR		; no interrupt from selection
	WSCSI	#$0c,#SPSCSIICR		; assert BSY and SEL
sel11:	clr.l	d0			; clear d0
	move.w	xdev(sp),d2		; d2.w = physical SCSI unit number
	bset	d2,d0			; set the appropriate bit for SCSI unit
	WSCSI	d0,#SPSCSIODR		; send the device number
	WSCSI	#5,#SPSCSIICR		; assert SEL and data bus
sel12:	RSCSI	#SPSCSIMR,d0		; read Mode Register
	andi.b	#$fe,d0			; clear arbitrate bit
	move.w	d0,WDC
sel13:	RSCSI	#SPSCSIICR,d0		; read Initiator Command Register
	andi.b	#$f7,d0			; clear BUSY bit
	move.w	d0,WDC
sel14:	nop				; deskew delays
	nop				; deskew delays

sel20:	TMOUT	#scstmout,d1		; set up timeout counter
	move.w	#SPSCSICR,WDL
sel21:	move.w	WDC,d0			; wait for bus to be busy
	btst	#6,d0
	bne.s	sel31			; received, to success
sel22:	CNTDN1	d1,sel21		; if not timed-out, wait some more

sel30:	move.l	#EREADF,d0		; timed out - read error
	WSCSI	#0,#SPSCSIICR		; clear SEL and data bus assertion
	bra	ret00
sel31:	WSCSI	#0,#SPSCSIICR		; clear SEL and data bus assertion

;****************************************
;* PREPARE AND SEND SCSI COMMAND BLOCK	*
;****************************************

cmd00:	lea	cmdblock,a0		; a0 -> beginning of command block
	move.b	#XREAD,(a0)+		; byte 0   = XREAD opcode
	clr.b	(a0)+			; byte 1   = unit number + reserved
	move.l	xlrecno(sp),(a0)+	; byte 2-5 = starting record number
	clr.b	(a0)+			; byte 6   = reserved
	move.w	xcount(sp),(a0)+	; byte 7-8 = number of sectors to read (msb-lsb)
	clr.b	(a0)			; byte 9   = reserved

cmd10:	WSCSI	#$02,#SPSCSITCR		; assert C/D
	WSCSI	#$01,#SPSCSIICR		; assert data bus
cmd11:	lea	cmdblock,a0		; a0 -> command block
	move.w	#LCMD-1,d2		; d2.w = number of bytes to send - 1

cmd20:	bsr	spw4mesg		; wait for REQ to come
	tst.l	d0			; timed out ?
	bmi.s	cmd30			; yes, so return
cmd21:	move.b	(a0)+,d0		; d0.b = byte to be sent
	WSCSI	d0,#SPSCSIDB		; write a byte out to data bus
cmd22:	bsr	spsndack		; assert ACK
	dbra	d2,cmd20		; until whole command block is sent
cmd23:	bra.s	cmd31

cmd30:	move.l	#EREADF,d0		; timed out - read error
	bra	ret00
cmd31:	nop

;****************************************
;* PREPARE PARAMETERS FOR SCSI READ	*
;*   d1.w = transfer length (in bytes)	*
;*   a0.l = buffer address		*
;****************************************

red00:	move.w	xcount(sp),d1		; d1.w = number of sectors to read
	move.l	xbuf(sp),a0		; a0.l = starting buffer address
red01:	bsr	_rcvspscsi		; read from SPARROW SCSI device
	tst.l	d0			; are there any read errors ?
red02:	beq.s	ret00			; no errors --> successful
red03:	move.l	#EREADF,d0		; read error

;****************************************
;* UNLOCK DMA CHIP AND RETURN STATUS	*
;****************************************

ret00:	RSCSI	#SPSCSIREI,d1		; clear potential interrupt

ret10:	move.w	#$180,WDL		; reset the chip for floppy
	RSTDELAY
	move.w	#$80,WDL		; Landon's code seems to presume we put $80 there
	RSTDELAY

ret20:	move.l	d0,-(sp)		; preserve d0.l
	move	sr,-(sp)		; go to IPL 7
ret21:	ori	#$700,sr		; no interrupts right now kudasai
	movecacrd0			; d0 = (cache control register)
	ori.w	#$808,d0		; dump both the D and I cache
	moved0cacr			; update cache control register
ret22:	move	(sp)+,sr		; restore interrupt state
	move.l	(sp)+,d0		; restore d0.l

ret23:	clr	flock			; NOW, signal that we are done
	movem.l	(sp)+,d2/a0		; restore registers
	rts

;********************************************************************************
;* RCVSPSCSI - send a SCSI command which receives data back			*
;*										*
;* Passed:	d1.w = transfer length						*
;*		a0.l = buffer address						*
;********************************************************************************

_rcvspscsi:

;****************************************
;* SET UP THE DMAC FOR DATA TRANSFER	*
;****************************************

	IF SPSCDMA=1

rcv00:	WSCSI	#0,#SPSCSIICR		; de-assert the data bus
	WSCSI	#1,#SPSCSITCR		; set data in phase
	RSCSI	#SPSCSIREI,d0		; clear potential interrupt

rcv01:	WSCSI	#2,#SPSCSIMR		; enable DMA mode

rcv02:	move.l	a0,-(sp)		; move it on stack
	move.b	3(sp),DMALOW		; set low-byte of address
	move.b	2(sp),DMAMID		; set mid-byte of address
	move.b	1(sp),DMAHI		; set high-byte of address
	addq.l	#$4,sp			; clean up stack

rcv03:	move.w	#$190,WDL		; toggle DMA chip for "receive"
	RSTDELAY			; delay
	move.w	#$090,WDL
	RSTDELAY			; delay

rcv04:	move.w	d1,WDC			; set DMA sector count

rcv05:	btst.b	#STA,WDSR		; wait till it's safe to access
	bne.s	rcv05			; the DMA channel

rcv06:	WSCSI	#0,#SPSCSIDIR		; start the DMA receive
	move.w	#0,WDL

rcv07:	bsr	spw4intr		; wait for interrupts
	tst.l	d0			; successful ?
	beq	rstat			; yes, go read status byte

rcv08:	rts

	ENDC	; SPSCDMA

;****************************************
;* NON-DMA DATA TRANSFER		*
;****************************************

dat00:	WSCSI	#0,#SPSCSIICR		; de-assert the data bus
	WSCSI	#1,#SPSCSITCR		; set data in phase
	RSCSI	#SPSCSIREI,d0		; clear potential interrupt

dat20:	TMOUT	#rcaltm,d1		; add time for recalibration
	move.w	#SPSCSICR,WDL
dat21:	move.w	WDC,d0			; read Control Register
	btst	#5,d0			; waiting for REQ to come
	bne.s	dat40			; if REQ comes, done
dat22:	CNTDN1	d1,dat21		; if there is more time, keep waiting

dat30:	move.l	#-1,d0			; return timed out
	rts

dat40:	RSCSI	#SPSCSIDSR,d0
	btst	#3,d0			; still in datain phase ?		
	beq.s	rstat			; if not, go get status

dat50:	RSCSI	#bSPSCSI,d0		; else read the next data byte
	move.b	d0,(a0)+

dat60:	bsr	spsndack
	bra.s	dat20			; do next byte

;****************************************
;* GET THE STATUS BYTE AND QUIT		*
;****************************************

rstat:
stt00:	WSCSI	#3,#SPSCSITCR		; status in phase
	RSCSI	#SPSCSIREI,d0		; clear potential interrupt

stt10:	bsr	spw4mesg		; wait for status byte
	tst.l	d0			; timed out ?
	beq.s	stt20			; no, send ack
	rts				; yes, return timed out

stt20:	RSCSI	#SPSCSIDB,d0		; get the status byte
	andi.w	#$ff,d0
	move.w	d0,stat			; save the status byte
stt21:	bsr	spsndack		; signal that status byte is here

stt30:	bsr	spw4mesg		; wait for message byte
	tst.l	d0			; timed out ?
	beq.s	stt31			; no, send ack
	rts				; yes, return timed out

stt31:	RSCSI	#SPSCSIDB,d0		; get and ignore message byte
stt32:	bsr	spsndack		; signal that message byte is here

stt40:	move.w	stat,d0			; recall the status byte
	rts

;********************************************************************************
;* SPW4INTR - wait for interrupts from 5380 or DMAC during DMA tranfers		*
;*										*
;*	When 5380 is interrupted, it indicates a change of data to		*
;*	  status phase (i.e., DMA is done), or ...				*
;*	When DMAC is interrupted, it indicates either DMA count is		*
;*	  zero, or there is an internal bus error.				*
;*										*
;* Returns:	d0.l = returned status or timeout error				*
;********************************************************************************

spw4intr:
int00:	TMOUT	#scxltmout+rcaltm,d1	; set up an extra long timeout including recalibration
int01:	btst	#5,GPIP			; interrupt ?
	beq.s	int03			; active LOW
	CNTDN1	d1,int01		; if there's more time, keep waiting
int02:	bra.s	int10			; go reset the SCSI bus

int03:	RSCSI	#SPSCSIREI,d1		; clear potential interrupt
	WSCSI	#0,#SPSCSIMR		; disable DMA mode
	WSCSI	#0,#SPSCSIICR		; make sure data bus is not asserted
	move.l	#0,d0			; return successful
	rts

int10:	btst.b	#STA,WDSR		; wait till it's safe to access
	bne.s	int10			; the DMA channel

int11:	move.w	#$190,WDL		; reset DMA chip
	RSTDELAY			; delay
	move.w	#$090,WDL
	RSTDELAY			; delay

int12:	movea.l	#_hz_200,a0		; a0.l = 200 hz clock

int20:	WSCSI	#$80,#SPSCSIICR		; reset SCSI bus - assert RST
int21:	move.l	#scstmout,d1		; d1.l = time out period
	add.l	(a0),d1			; d1.l = current time + # clicks to wait
int22:	cmp.l	(a0),d1			; has timed out yet
	bhi.s	int22			; no, continue waiting

int30:	WSCSI	#0,#SPSCSIICR		; clear RST
int31:	move.l	#scltmout,d1		; d1.l = time out period
	add.l	(a0),d1			; d1.l = current time + # clicks to wait
int32:	cmp.l	(a0),d1			; has timed out yet
	bhi.s	int32			; no, continue waiting

int40:	move.l	#-1,d0			; return time out
	rts

;********************************************************************************
;* spw4mesg()	- wait for REQ to come during hand shake of non-data bytes	*
;*										*
;* Returns	: 0 - if successful, -1 - times out				*
;********************************************************************************

spw4mesg:
msg00:	TMOUT	#scstmout,d1		; set up timeout counter
	move.w	#SPSCSICR,WDL
msg01:	move.w	WDC,d0			; read Control Register
	btst	#5,d0			; waiting for REQ to come
	bne.s	msg03			; if REQ comes, done
	CNTDN1	d1,msg01		; if there is more time, keep waiting
msg02:	move.l	#-1,d0			; return timed out
	rts
msg03:	move.l	#0,d0			; return successful
	rts

;********************************************************************************
;* SPSNDACK - assert ACK - always successful					*
;********************************************************************************

spsndack:
ack00:	move.w 	#$ff,d1			; set the time out counter

ack10:	RSCSI	#SPSCSIICR,d0		; read Initiator Command Register
	ori.b	#$11,d0			; assert ACK (and data bus)
	move.w	d0,WDC

ack20:	move.w	#SPSCSITCR,WDL
ack21:	move.w	WDC,d0			; move the target command register to d0
	btst	#3,d0			; waiting for ??? to come
	bne.s	ack30			; if ??? comes, done
	dbra	d1,ack21		; loop and wait for ??? to come

ack30:	RSCSI	#SPSCSIICR,d0		; read Initiator Command Register
	andi.b	#$ef,d0			; clear ACK
	move.w	d0,WDC
ack40:	rts

;******************************************************************************
;*	DATA SECTION
;******************************************************************************

	CSECT	bbbb,bss

;****************************************
;* DATA VARIABLES USED HERE		*
;****************************************

cmdblock:	ds.b	10		; command block
stat:		ds.w	1		; status byte

;******************************************************************************
;* END OF PRAGRAM
;******************************************************************************

	END

;********************************************************************************
;* End of MINE.S								*
;********************************************************************************
