/******************************************************************************/
/* ST_GLOB.C								      */
/******************************************************************************/

#include "st_def.h"
#include "st_struc.h"
#include "st_about.h"
#include "st_aes.h"
#include "st_bind.h"
#include "st_key.h"
#include "st_lib.h"
#include "st_pref.h"
#include "st_scrap.h"
#include "st_tos.h"
#include "cd_bind.h"

#include "cdmonusa.h"

/* -------------------------------------------------------------------------- */

extern	int	gl_mint_installed;

extern	GRECT	gl_windrect;

/* -------------------------------------------------------------------------- */

unsigned long	_STACK = 16384l;
unsigned long	_STKDELTA = 128l;

int	_VDIcontrl[20];
int	_VDIintin[256];
int	_VDIptsin[512];
int	_VDIintout[256];
int	_VDIptsout[512];

/* -------------------------------------------------------------------------- */

int	gl_appid;			/* the application id. */

int	gl_workin[] = { 1,1,1,1,1,1,1,1,1,1,2 };
int	gl_workout[57];

int	gl_scrhandle = 0;		/* screen vdi handle */

int	gl_nocolors;			/* number of colours */

int	gl_noplanes;			/* number of planes in video display */
int	gl_currez;			/* current resolution */

int	gl_boxwide;			/* system default box wide (menu wide etc.) */
int	gl_boxhite;			/* system default box hite (menu hite etc.) */

int	gl_charhite; 			/* default system cell height in rasters */
int	gl_charwide; 			/* default system cell width in rasters */

int	gl_sysfont;			/* default system font id */
int	gl_syssize;			/* default system font size in points */

int	gl_scrbar[4];			/* full screen coordinates */
GRECT	gl_scrbox;			/* total screen rectangle */

GRECT	gl_fullbox;			/* full screen box */

int	gl_sysraswide;			/* raster width as returned by open workstation call */
int	gl_sysrashite;			/* raster height as returned by open workstation call */

double	gl_raswide;			/* screen pixel width in inches */
double	gl_rashite;			/* screen pixel height in inches */
int	gl_rasinch;			/* # of rasters in an inch */

double	gl_inch2micron;			/* inch to micron factor */

int	gl_ispanel = FALSE;		/* if top window is non-sutra window */

FDB	gl_nullfdb =			/* for screen ras-op */
		{ 0L,0,0,0,0,0,0,0,0 };

unsigned char	gl_pathname[ MAX_PATHNAME ];
unsigned char	gl_sysname[] = { "CDROMSYS\\" };

unsigned char	*gl_untitledstr;	/* pointer to a string "Untitled" */

/* -------------------------------------------------------------------------- */

OBJECT	*gl_actmenu;		/* menu address */

int	gl_acthandle;		/* handle of the emulator window */
CDROM	*gl_actdocu;		/* global pointer to current CDROM structure */

/* -------------------------------------------------------------------------- */

static	int	junk = 0;		/* a general purpose junk variable */

static	unsigned char	*gl_extension =	/* file extension type */
		{ ".TOC" };

static	int	gl_initerrno;		/* error number if initialisation fails */

static	int	gl_windoptions =	/* different window gadgets */
		NAME | MOVER | CLOSER | FULLER | SIZER | UPARROW |
		DNARROW | LFARROW | RTARROW | HSLIDE | VSLIDE;

static	unsigned char	gl_windtitle[ MAX_TITLENAME ];

/* -------------------------------------------------------------------------- */

static	int	gl_mint_aesversion;
static	int	gl_mint_multiaes;
static	int	gl_mint_maxcharhite;
static	int	gl_mint_maxcharwide;

static	int	gl_isiconified = FALSE;

static	int	gl_minwinhite;

/* -------------------------------------------------------------------------- */

static	int	st_open_application( void );
static	int	st_init_application( void );
static	void	st_close_application( void );

static	void	st_handle_timer( void );
static	void	st_handle_message( int * );
static	void	st_handle_button( int, int, int, int );
static	void	st_handle_keybd( unsigned int );

static	void	st_menu_handle( int * );

static	void	st_wind_redraw( int * );
static	void	st_wind_activate( int * );
static	void	st_wind_close( int * );
static	void	st_wind_full( int * );
static	void	st_wind_size( int * );
static	void	st_wind_move( int * );
static	void	st_wind_arrowed( int * );
static	void	st_wind_hslid( int * );
static	void	st_wind_vslid( int * );
static	void	st_wind_bottomed( int * );
static	void	st_wind_iconify( int * );
static	void	st_wind_uniconify( int * );

static	int	st_docuclose( void );
static	void	st_func_quit( void );

static	int	st_newopen_init( void );
static	void	st_func_new( void );
static	void	st_func_open( void );
static	void	st_func_save( void );
static	void	st_func_saveas( void );
static	int	st_docusave( void );
static	int	st_docusaveas( void );

static	int	st_create_window( void );
static	void	st_destroy_window( void );
static	void	st_setwindtitle( void );

static	int	st_mint_dispicon( GRECT * );

/* -------------------------------------------------------------------------- */

void	main( int, unsigned char ** );

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * First get the screen handle by making graf_handle call and then open the
 * virtual work station for the application.
 *
 * returns 1 if just quit,
 * returns 2 if application exit to be done,
 * returns 3 if workstation to be closed,
 * returns 0 on success
 */

static	int	st_open_application()
{
register unsigned char	*cp;
unsigned char	command[ MAX_PATHNAME ];
unsigned char	tail[ MAX_PATHNAME ];

/* call application intialisation routine and get the application id */

	gl_appid = appl_init();
	if (gl_appid == -1)
	{
		st_error_appl_init();
		return(1);
	}
	st_mouse_busy();

/* initialise the system font id */

	gl_sysfont = 1;		/* <--- MUST BE BEFORE installing fonts */

/* check for various system specifications */

	st_get_tos_version();		/* get the tos version */
	st_get_language();		/* get the current system language settings */

	st_ismint_installed();		/* check if running under MiNT */

/* initialise the variables used for mint specific stuff */

	gl_mint_aesversion  = _AESglobal[0];
	gl_mint_multiaes    = ((_AESglobal[1] == -1) ? TRUE : FALSE);
	gl_mint_maxcharhite = _AESglobal[13];
	gl_mint_maxcharwide = _AESglobal[14];

	if (gl_mint_aesversion >= VERSION_AES_MULTITOS400)
	{
		gl_windoptions |= SMALLER;
	}

/* call shell_read() to get the default pathname */
/* this pathname is used everywhere as the pathname for the CD emulator */

	strcpy(gl_pathname, "");
	if (shel_read(command, tail) && (cp = strrchr(command, '\\')))
	{
		cp[1] = '\0';
		strcpy(gl_pathname, command);
	}

/* check if the above routines got any pathname, if not then get the current */
/* pathname and assign it to the pathname where WORKS is being run from */

	if (!strlen(gl_pathname))
	{
		st_get_curpath(gl_pathname);
	}

/* load the alert and free strings from the alternate resource */

	if (!st_load_freestrs())
	{
		return(2);
	}

/* get the AES handle and the system character cell width and height */

	gl_scrhandle = graf_handle(&gl_charwide, &gl_charhite, &gl_boxwide, &gl_boxhite);

/* open virtual work station for screen updates */

	gl_currez = Getrez();

	gl_workin[0] = gl_currez + 2;
	v_opnvwk(gl_workin, &gl_scrhandle, gl_workout);

/* number of colors supported */

	gl_nocolors = gl_workout[13];

/* get the screen box as returned by v_opnvwk() */

	gl_scrbox.g_x = 0;
	gl_scrbox.g_y = 0;
	gl_scrbox.g_w = gl_workout[0] + 1;
	gl_scrbox.g_h = gl_workout[1] + 1;

	gl_scrbar[0] = 0;
	gl_scrbar[1] = 0;
	gl_scrbar[2] = gl_workout[0];
	gl_scrbar[3] = gl_workout[1];

/*
 * get the size of the largest possible window
 *
 * in case of MiNT wind_get(DESKTOP, WF_FULLXYWH, ...) call returns a 1 pixel
 * offset on each side of the actual desktop window.  St we try to calculate
 * the full window size in a different way
 */
	gl_fullbox.g_x = 0;
	gl_fullbox.g_y = gl_boxhite;
	gl_fullbox.g_w = gl_scrbox.g_w;
	gl_fullbox.g_h = gl_scrbox.g_h - gl_boxhite;

/* if current resolution supports less than 80 character columns then error */
/*
 *	if ((gl_scrbox.g_w / gl_charwide) < 80)
 *	{
 *		st_error_message(E_LOWREZ);
 *		return(3);
 *	}
 */

/*
 * assuming 640 pixels per 7 inch (ie apprx 91 dpi) according to
 * Atari Development Partner Newsletter, May 1990 issue.  Also,
 * the width and height of a pixel as returned by v_opnvwk() gives
 * the aspect ratio.  so we can calculate the pixel height in inches
 */
	gl_raswide = ((double)7 / (double)640);
	gl_rashite = ((double)gl_workout[4] / (double)gl_workout[3]) * gl_raswide;
	gl_rasinch = (int)((1.0 / gl_raswide) + 0.5);

/* inch to micron conversion factor */

	gl_inch2micron = (double)(10000.0 / 0.393701);

	gl_sysraswide = (int)(gl_raswide * gl_inch2micron);
	gl_sysrashite = (int)(gl_rashite * gl_inch2micron);

/* get the number of planes in the current resolution */

	vq_extnd(gl_scrhandle, 1, gl_workout);
	gl_noplanes = gl_workout[4];
	vq_extnd(gl_scrhandle, 0, gl_workout);

	return(0);
}

/* -----------------------------------------------------------------------------
 * returns 3 if only work station to be closed,
 * returns 4 if resource to be unloaded,
 * returns 5 if fonts to be unloaded,
 * returns 0 on success.
 */

static	int	st_init_application()
{
int	hite;

/* load the resources depending upon no of colors supported by system */

	st_mouse_busy();
	st_info_showstatus(DIAL_INFO_LOAD_RESOURCE);

	if (!st_load_resource((gl_nocolors < 16) ? 0 : 1))
	{
		st_error_load_resource();
		return(3);
	}

/*
 * default resource is for high resolution (monochrome), so set different
 * images and icons in the resource file for the st medium resolution
 */
	st_init_medicons();
	gl_untitledstr = st_freestr(FR_UNTITLED);

/* get pointers to menus and initialise the main menu */

	gl_actmenu = st_objc_get_tree(MAINMENU);

	st_info_showstatus(DIAL_INFO_INITIALISING);

/* set the default system face and size in points */

	vst_font(gl_scrhandle, gl_sysfont);
	for (gl_syssize = 0 ;; gl_syssize ++)
	{
		vst_point(gl_scrhandle, gl_syssize, &junk, &junk, &junk, &hite);
		if (hite == gl_charhite)
			break;
	}

/* initialise document header and the clipboard document */

	if (!st_clip_initscrap())
	{
		st_error_message(E_SCRAPINIT);
		return(5);
	}

/*
 * load the default preferences - load preferences from the INF file;
 * check if the spelling checter exists, if not then set it to the
 * current spelling checker for the current country language settings
 */
	st_defl_preference();
	if (!st_load_preference())
	{
		st_defl_preference();
	}

/* perform initialisation for different modules in st-sutra */

	gl_minwinhite = cd_initialise();
	if (!gl_minwinhite)
	{
		form_error(8);
		return(5);
	}

/* initialise document header and window title string pointers */

	gl_acthandle = -1;
	gl_actdocu = cd_create();
	if (gl_actdocu == NULL)
	{
		form_error(8);
		return(5);
	}
	cd_set_parameters();

/* create, open and redraw the window for the emulator */

	if (!st_create_window())
	{
		st_error_message(E_WINDLIM);
		return(5);
	}
	st_redraw_window();
	st_set_menu();
	return(0);
}

/* -----------------------------------------------------------------------------
 * gl_initerrno = 0, if success.
 * gl_initerrno = 1, if just quit,
 * gl_initerrno = 2, if application exit to be done,
 * gl_initerrno = 3, if only work station to be closed,
 * gl_initerrno = 4, if resource to be unloaded,
 * gl_initerrno = 5, if fonts to be unloaded,
 */

static	void	st_close_application()
{
register int	errflag = gl_initerrno;

	if (errflag >= 4)	rsrc_free();
	if (errflag >= 3)	v_clsvwk(gl_scrhandle);
	if (errflag >= 2)	appl_exit();
	exit(0);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * initialises system and ST Sutra parameters and call the global event routine
 */

void	main(argc, argv)
int	argc;
unsigned char	*argv[];
{
static	int	mousex, mousey, bstate, kstate, times;
static	int	key, buffer[8];
static	int	handle;
static	int	happened;

	if (argc > 2)
	{
		printf("\nUsage : emulator [filename]\n");
		exit(0);
	}

/* open the virtual screen work station and perform initialisation. */
/* display the st-sutra information box as initial screen. */
/* do the rest of the initialisation. */
/* erase the st-sutra information box from the screen. */

	gl_initerrno = st_open_application();
	if (gl_initerrno)
	{
		st_close_application();
	}

	wind_update(BEG_UPDATE);
	st_info_show();

	gl_initerrno = st_init_application();

	st_info_erase();
	wind_update(END_UPDATE);

	if (gl_initerrno)
	{
		st_close_application();
	}

/* set the init error number to 5 so that a proper close is done at quit */

	gl_initerrno = 5;

/* display the main menu, load the file if a filename is passed to sutra,
/* otherwise display the document selector box */

	wind_update(BEG_UPDATE);
	menu_bar(gl_actmenu, 1);
	graf_mkstate(&junk, &junk, &junk, &junk);

	if (argc > 1)
	{
		st_load_file(argv[1]);
	}
	else
	{
		st_func_open();
	}

	st_mouse_free();
	wind_update(END_UPDATE);

/************************************************/
/* Global event tracking to handle user actions */
/************************************************/

	for (;;)
	{
		happened = evnt_multi((MU_MESAG | MU_KEYBD | MU_BUTTON | MU_TIMER),
					2, 1, 1,
					0, 0,0,0,0, 0, 0,0,0,0, 
					buffer, 500, 0, 
					&mousex, &mousey, &bstate, &kstate, &key, &times);

		wind_get(0, WF_TOP, &handle, &junk, &junk, &junk);
		if ((handle >= 0) && (handle != gl_acthandle))
		{
			if (!gl_ispanel)
			{
				gl_ispanel = TRUE;
				st_set_menu();
			}
		}
		else
		{
			if (gl_ispanel)
			{
				gl_ispanel = FALSE;
				st_set_menu();
			}
		}

		if (happened & (MU_TIMER | MU_MESAG | MU_BUTTON | MU_KEYBD))
		{
			wind_update(BEG_UPDATE);
			if (happened & MU_TIMER)	st_handle_timer();
			if (happened & MU_MESAG)	st_handle_message(buffer);
			if (happened & MU_BUTTON)	st_handle_button(mousex, mousey, kstate, times);
			if (happened & MU_KEYBD)	st_handle_keybd(key);
			wind_update(END_UPDATE);
		}
	}
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	void	st_handle_timer()
{
	return;
}

/* -----------------------------------------------------------------------------
 * message event handler
 */

static	void	st_handle_message(buffer)
register int	buffer[];
{
	switch (buffer[0])
	{
		case	MN_SELECTED    : st_menu_handle(buffer);	return;

		case	WM_REDRAW      : st_wind_redraw(buffer);	return;
		case	WM_TOPPED      : st_wind_activate(buffer);	return;

		case	WM_FULLED      : st_wind_full(buffer);		return;
		case	WM_SIZED       : st_wind_size(buffer);		return;
		case	WM_MOVED       : st_wind_move(buffer);		return;

		case	WM_ARROWED     : st_wind_arrowed(buffer);	return;
		case	WM_HSLID       : st_wind_hslid(buffer);		return;
		case	WM_VSLID       : st_wind_vslid(buffer);		return;

		case	WM_CLOSED     : st_wind_close(buffer);		return;

		case	WM_BOTTOMED   : st_wind_bottomed(buffer);	return;

		case	WM_ICONIFY    : st_wind_iconify(buffer);	return;
		case	WM_UNICONIFY  : st_wind_uniconify(buffer);	return;
		case	WM_ALLICONIFY : st_wind_uniconify(buffer);	return;
	}
	return;
}

/* -----------------------------------------------------------------------------
 * global button event handler
 */

static	void	st_handle_button(mx, my, kstate, times)
register int	mx, my, kstate, times;
{
GRECT	a;

/* if emulator window is not on top then return */

	if (gl_ispanel)				return;

/* if the window iconified return */

	if (gl_isiconified)			return;

/* Also if click outside the active window work area then return */

	wind_get(gl_acthandle, WF_WORKXYWH, &a.g_x, &a.g_y, &a.g_w, &a.g_h);

	if (mx < a.g_x)				return;
	if (my < a.g_y)				return;
	if (mx > (a.g_x + a.g_w - 1))		return;
	if (my > (a.g_y + a.g_h - 1))		return;

/* Handle the mouse button event */

	cd_handle_button(mx, my, kstate, times);
	st_mouse_free();
	return;
}

/* -----------------------------------------------------------------------------
 * key board event handler
 */

static	void	st_handle_keybd(key)
register unsigned int	key;
{
static	unsigned int	gl_keybd_items[][3] =
	{
	SHFT_UP		,	-1	,	-1		,
	SHFT_DOWN	,	-1	,	-1		,

	KEY_HOME	,	-1	,	-1		,
	SHFT_HOME	,	-1	,	-1		,

	KEY_UP		,	-1	,	-1		,
	KEY_DOWN	,	-1	,	-1		,

	KEY_INS		,	-1	,	-1		,

	CTRL_N		,	CD_FILE	,	CD_NEW		,
	CTRL_O		,	CD_FILE	,	CD_OPEN		,
	CTRL_S		,	CD_FILE	,	CD_SAVE		,
	CTRL_SHFT_S	,	CD_FILE	,	CD_SAVAS	,
	CTRL_L		,	CD_FILE	,	CD_LOGFL	,
	CTRL_E		,	CD_FILE	,	CD_EMULT	,
	CTRL_Q		,	CD_FILE	,	CD_QUIT		,

	KEY_UNDO	,	CD_EDIT	,	CD_UNDO		,
	CTRL_X		,	CD_EDIT	,	CD_CUT		,
	CTRL_C		,	CD_EDIT	,	CD_COPY		,
	CTRL_V		,	CD_EDIT	,	CD_PASTE	,
	KEY_DEL		,	CD_EDIT	,	CD_DELET	,
	KEY_F3		,	CD_EDIT	,	CD_INSSS	,
	KEY_F2		,	CD_EDIT	,	CD_INSTR	,
	KEY_F1		,	CD_EDIT	,	CD_INSFL	,
	CTRL_A		,	CD_EDIT	,	CD_SELAL	,
	KEY_F5		,	CD_EDIT	,	CD_COMNT	,
	KEY_F4		,	CD_EDIT	,	CD_EDFIL	,

	KEY_F8		,	CD_SRCH	,	CD_GOSES	,
	KEY_F9		,	CD_SRCH	,	CD_GOTRK	,
	KEY_F10		,	CD_SRCH	,	CD_FIND		,
	SHFT_F10	,	CD_SRCH	,	CD_FINDN	,
	-1		,	-1	,	-1
	};

static	void	(*gl_keybd_functable[])( void ) =
	{
	/* SHFT_UP	*/	cd_key_pageup		,
	/* SHFT_DOWN	*/	cd_key_pagedown		,

	/* KEY_HOME	*/	cd_key_home		,
	/* SHFT_HOME	*/	cd_key_end		,

	/* KEY_UP	*/	cd_key_up		,
	/* KEY_DOWN	*/	cd_key_down		,

	/* KEY_INS	*/	cd_key_select		,

	/* CTRL_N	*/	st_func_new		,
	/* CTRL_O	*/	st_func_open		,
	/* CTRL_S	*/	st_func_save		,
	/* CTRL_SHFT_S	*/	st_func_saveas		,
	/* CTRL_L	*/	cd_set_logfilename	,
	/* CTRL_E	*/	cd_comnd_emulate	,
	/* CTRL_Q	*/	st_func_quit		,

	/* KEY_UNDO	*/	cd_undo			,
	/* CTRL_X	*/	cd_cut			,
	/* CTRL_C	*/	cd_copy			,
	/* CTRL_V	*/	cd_paste		,
	/* KEY_DEL	*/	cd_delete		,
	/* KEY_F3	*/	cd_insert_session	,
	/* KEY_F2	*/	cd_insert_track		,
	/* KEY_F1	*/	cd_insert_file		,
	/* CTRL_A	*/	cd_select_all		,
	/* KEY_F5	*/	cd_add_comments		,
	/* KEY_F4	*/	cd_edit_filename	,

	/* KEY_F8	*/	cd_goto_session		,
	/* KEY_F9	*/	cd_goto_track		,
	/* KEY_F10	*/	cd_find			,
	/* SHFT_F10	*/	cd_find_next		
	};

register OBJECT	*Tree = gl_actmenu;
register int	item;
register int	i;

	key = st_key_process(key);
	if (!(key & KbEXTND))
		return;

	for (i = 0 ; gl_keybd_items[i][0] != -1 ; i++)
	{
		if (gl_keybd_items[i][0] != key)		continue;

		item = gl_keybd_items[i][2];
		if (item != -1)
			{ if (Tree[ item ].ob_state & DISABLED)	break; }
		else	{ if (gl_ispanel)			break; }

		item = gl_keybd_items[i][1];
		if (item != -1)	menu_tnormal(Tree, item, 0);

		(*gl_keybd_functable[i])();

		if (item != -1)	menu_tnormal(Tree, item, 1);

		st_clip_freeafterpaste();
		st_mouse_free();
		break;
	}
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	void	st_menu_handle(buffer)
register int	buffer[];
{
static	int	gl_menu_items[] =
	{
	CD_ABOUT	,	/* st_info_dialog()		*/

	CD_NEW		,	/* st_func_new()		*/
	CD_OPEN		,	/* st_func_open()		*/
	CD_SAVE		,	/* st_func_save()		*/
	CD_SAVAS	,	/* st_func_saveas()		*/
	CD_FILST	,	/* cd_save_filelist()		*/
	CD_LOGFL	,	/* cd_set_logfilename()		*/
	CD_EMULT	,	/* cd_comnd_emulate()		*/
	CD_QUIT		,	/* st_func_quit()		*/

	CD_UNDO		,	/* cd_undo()			*/
	CD_CUT		,	/* cd_cut()			*/
	CD_COPY		,	/* cd_copy()			*/
	CD_PASTE	,	/* cd_paste()			*/
	CD_DELET	,	/* cd_delete()			*/
	CD_INSSS	,	/* cd_insert_session()		*/
	CD_INSTR	,	/* cd_insert_track()		*/
	CD_INSFL	,	/* cd_insert_file()		*/
	CD_SELAL	,	/* cd_select_all()		*/
	CD_COMNT	,	/* cd_add_comments()		*/
	CD_EDFIL	,	/* cd_edit_filename()		*/

	CD_GOSES	,	/* cd_goto_session()		*/
	CD_GOTRK	,	/* cd_goto_track()		*/
	CD_FIND		,	/* cd_find()			*/
	CD_FINDN	,	/* cd_find_next()		*/

	CD_ROMNM	,	/* cd_dial_cdromtitle()		*/
	CD_PREF		,	/* cd_dial_preferences()	*/
	CD_PRELD	,	/* cd_dial_preloadpref()	*/
	CD_LTNCY	,	/* cd_dial_latency()		*/
	-1
	};

static	void	(*gl_menu_functable[])( void ) =
	{
	/* CD_ABOUT	*/	st_info_dialog		,

	/* CD_NEW	*/	st_func_new		,
	/* CD_OPEN	*/	st_func_open		,
	/* CD_SAVE	*/	st_func_save		,
	/* CD_SAVAS	*/	st_func_saveas		,
	/* CD_FILST	*/	cd_save_filelist	,
	/* CD_LOGFL	*/	cd_set_logfilename	,
	/* CD_EMULT	*/	cd_comnd_emulate	,
	/* CD_QUIT	*/	st_func_quit		,

	/* KEY_UNDO	*/	cd_undo			,
	/* CD_CUT	*/	cd_cut			,
	/* CD_COPY	*/	cd_copy			,
	/* CD_PASTE	*/	cd_paste		,
	/* CD_DELET	*/	cd_delete		,
	/* CD_INSSS	*/	cd_insert_session	,
	/* CD_INSTR	*/	cd_insert_track		,
	/* CD_INSFL	*/	cd_insert_file		,
	/* CD_SELAL	*/	cd_select_all		,
	/* CD_COMNT	*/	cd_add_comments		,
	/* CD_EDFIL	*/	cd_edit_filename	,

	/* CD_GOSES	*/	cd_goto_session		,
	/* CD_GOTRK	*/	cd_goto_track		,
	/* CD_FIND	*/	cd_find			,
	/* CD_FINDN	*/	cd_find_next		,

	/* CD_ROMNM	*/	cd_dial_cdromtitle	,
	/* CD_PREF	*/	cd_dial_preferences	,
	/* CD_PRELD	*/	cd_dial_preloadpref	,
	/* CD_LTNCY	*/	cd_dial_latency
	};

register OBJECT	*Tree = gl_actmenu;
register int	i;

	for (i = 0 ; gl_menu_items[i] != -1 ; i++)
	{
		if (gl_menu_items[i] != buffer[4])
			continue;

		if (Tree[ buffer[4] ].ob_state & DISABLED)
			break;

		(*gl_menu_functable[i])();

		st_clip_freeafterpaste();
		st_mouse_free();
		break;
	}
	menu_tnormal(Tree, buffer[3], 1);
	return;
}

/* -----------------------------------------------------------------------------
 * redraws a window according to the redraw message received
 */

static	void	st_wind_redraw(buffer)
register int	buffer[];
{
register int	handle = buffer[3];
GRECT	drawrect;
GRECT	box;

	if (handle != gl_acthandle)
		return;

	drawrect.g_x = buffer[4];
	drawrect.g_y = buffer[5];
	drawrect.g_w = buffer[6];
	drawrect.g_h = buffer[7];

	wind_get(handle, WF_FIRSTXYWH, &box.g_x, &box.g_y, &box.g_w, &box.g_h);
	while (box.g_w && box.g_h)
	{
		if (rc_intersect(&drawrect, &box) && !st_mint_dispicon(&box))
		{
			cd_redraw(&box);
		}
		wind_get(handle, WF_NEXTXYWH, &box.g_x, &box.g_y, &box.g_w, &box.g_h);
	}
	return;
}

/* -----------------------------------------------------------------------------
 * Called whenever a window is activated by mouse click
 */

static	void	st_wind_activate(buffer)
register int	buffer[];
{
register int	handle = buffer[3];

	if (handle != gl_acthandle)
		return;

	wind_set(handle, WF_TOP, junk, junk, junk, junk);
	gl_ispanel = FALSE;
	st_set_menu();
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	st_wind_close(buffer)
register int	buffer[];
{
register int	handle = buffer[3];

	if (handle != gl_acthandle)
		return;

	st_func_quit();
	return;
}

/* -----------------------------------------------------------------------------
 * Full/small (previous size) the current window
 */

static	void	st_wind_full(buffer)
register int	buffer[];
{
register int	handle = buffer[3];
GRECT	b,a;

	if (handle != gl_acthandle)
		return;

	a.g_x = gl_fullbox.g_x;
	a.g_y = gl_fullbox.g_y;
	a.g_w = gl_fullbox.g_w;
	a.g_h = gl_fullbox.g_h;
	wind_get(handle, WF_CURRXYWH, &b.g_x, &b.g_y, &b.g_w, &b.g_h);

	if ((b.g_x == a.g_x) && (b.g_y == a.g_y) && (b.g_w == a.g_w) && (b.g_h == a.g_h))
	{
		wind_get(handle, WF_PREVXYWH, &a.g_x, &a.g_y, &a.g_w, &a.g_h);
		graf_shrinkbox(a.g_x, a.g_y, a.g_w, a.g_h, b.g_x, b.g_y, b.g_w, b.g_h);
	}
	else
	{
		graf_growbox(b.g_x, b.g_y, b.g_w, b.g_h, a.g_x, a.g_y, a.g_w, a.g_h);
	}
	wind_set(handle, WF_CURRXYWH, a.g_x, a.g_y, a.g_w, a.g_h);
	cd_set_windareas();
	return;
}

/* -----------------------------------------------------------------------------
 * Size the current window
 */

static	void	st_wind_size(buffer)
register int	buffer[];
{
register int	handle = buffer[3];

	if (handle != gl_acthandle)
		return;

	if (buffer[7] < gl_minwinhite)	buffer[7] = gl_minwinhite;

	wind_set(handle, WF_CURRXYWH, buffer[4], buffer[5], buffer[6], buffer[7]);
	cd_set_windareas();
	return;
}

/* -----------------------------------------------------------------------------
 * Move the current window
 */

static	void	st_wind_move(buffer)
register int	buffer[];
{
register int	handle = buffer[3];

	if (handle != gl_acthandle)
		return;

	wind_set(handle, WF_CURRXYWH, buffer[4], buffer[5], buffer[6], buffer[7]);
	cd_set_windareas();
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	st_wind_arrowed(buffer)
register int	buffer[];
{
register int	handle = buffer[3];

	if (handle != gl_acthandle)
		return;

	cd_wind_arrowed(buffer[4]);
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	st_wind_hslid(buffer)
register int	buffer[];
{
register int	handle = buffer[3];

	if (handle != gl_acthandle)
		return;

	cd_wind_hslide(buffer[4]);
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	st_wind_vslid(buffer)
register int	buffer[];
{
register int	handle = buffer[3];

	if (handle != gl_acthandle)
		return;

	cd_wind_vslide(buffer[4]);
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	st_wind_bottomed(buffer)
register int	buffer[];
{
register int	handle = buffer[3];

	if (handle != gl_acthandle)
		return;

	if (gl_mint_aesversion < VERSION_AES_FALCONTOS)
		return;

	wind_set(handle, WF_BOTTOM, 0, 0, 0, 0);
	gl_ispanel = TRUE;
	st_set_menu();
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	st_wind_iconify(buffer)
register int	buffer[];
{
register int	handle = buffer[3];
GRECT	b;

	if (handle != gl_acthandle)
		return;

	if (gl_mint_aesversion < VERSION_AES_MULTITOS400)
		return;

	wind_get(handle, WF_CURRXYWH, &b.g_x, &b.g_y, &b.g_w, &b.g_h);
	graf_shrinkbox(b.g_x, b.g_y, b.g_w, b.g_h, buffer[4], buffer[5], buffer[6], buffer[7]);
	wind_set(handle, WF_ICONIFY, buffer[4], buffer[5], buffer[6], buffer[7]);

	gl_isiconified = TRUE;
	st_set_menu();
	cd_set_windareas();
	return;
}

/* -----------------------------------------------------------------------------
 */

static	void	st_wind_uniconify(buffer)
register int	buffer[];
{
register int	handle = buffer[3];
GRECT	b;

	if (handle != gl_acthandle)
		return;

	if (gl_mint_aesversion < VERSION_AES_MULTITOS400)
		return;

	wind_get(handle, WF_CURRXYWH, &b.g_x, &b.g_y, &b.g_w, &b.g_h);
	graf_growbox(b.g_x, b.g_y, b.g_w, b.g_h, buffer[4], buffer[5], buffer[6], buffer[7]);
	wind_set(handle, WF_UNICONIFY, buffer[4], buffer[5], buffer[6], buffer[7]);

	gl_isiconified = FALSE;
	st_set_menu();
	cd_set_windareas();
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	int	st_docuclose()
{
register CDROM	*docu = gl_actdocu;
register int	response;
unsigned char	s[ MAX_STRLEN ];

	if (!(docu->Settings & GL_DOCUNOTSAVED))
		return(TRUE);

	st_str_ncpy(s, st_str_filename(gl_windtitle), 29);
	response = st_error_message_one(E_SAVEDOCU, (long)s);
	if (response == 3)
		return(FALSE);

	if (response == 1)
	{
		if (!st_docusave())
			return(FALSE);
	}
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 * Quit CD ROM Emulator.
 */

static	void	st_func_quit()
{
	if (!st_docuclose())
		return;

	st_mouse_busy();
	cd_free(gl_actdocu);
	st_save_preference();
	st_destroy_window();
	menu_bar(gl_actmenu, 0);
	st_mouse_free();
	wind_update(END_UPDATE);
	st_close_application();
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	int	st_newopen_init()
{
int	buffer[8];

	if (gl_ispanel)
	{
		buffer[3] = gl_acthandle;
		st_wind_activate(buffer);
	}
	return(st_docuclose());
}

/* -----------------------------------------------------------------------------
 * Create a new document.
 */

static	void	st_func_new()
{
register CDROM	*docu;

	if (!st_newopen_init())
		return;

	st_mouse_busy();
	docu = cd_create();
	if (docu == NULL)
	{
		st_error_message(E_MALL);
		return;
	}

	cd_undo_free();
	cd_free(gl_actdocu);
	gl_actdocu = docu;
	st_setwindtitle();
	cd_set_parameters();
	st_redraw_window();

	cd_dial_cdromtitle();
	st_set_menu();
	return;
}

/* -----------------------------------------------------------------------------
 * Create a new document. Load an existing file as ST Sutra document
 */

static	void	st_func_open()
{
static	unsigned char	gl_deffilename[ MAX_FILENAME ];

unsigned char	filename[ MAX_PATHNAME ];

	if (!st_newopen_init())
		return;

/* get the default file name */

	strcpy(filename, gl_deffilename);

/* get the filename to open from user */

	if (!st_get_filename(0, AREAD, filename, gl_extension, FSEL_LOAD_CDROM))
		return;

/* save the default filename */

	st_mouse_busy();
	strcpy(gl_deffilename, st_str_filename(filename));
	st_load_file(filename);
	return;
}

/* -----------------------------------------------------------------------------
 * loads an existing file : name of the file is passed as parameter
 */

int	st_load_file(filename)
register unsigned char	*filename;
{
register CDROM	*docu;
register FILE	*fp;
int	ret;

	fp = fopen(filename, "rb");
	if (fp == NULL)
	{
		st_error_message(E_OPENF);
		return(FALSE);
	}

	mem_set_load_alloc();
	docu = cd_load(fp, &ret);
	fclose(fp);
	mem_reset_load_alloc();

	if (ret)
	{
		cd_free(docu);
		st_error_message((ret == 1) ? E_MALL : E_READF);
		return(FALSE);
	}

	docu->Settings &= ~(GL_DOCUNEW | GL_DOCUNOTSAVED);
	strcpy(docu->PathName, filename);

	cd_undo_free();
	cd_free(gl_actdocu);
	gl_actdocu = docu;
	st_setwindtitle();
	cd_set_parameters();
	st_redraw_window();

	st_set_menu();
	return(TRUE);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Save the current document.
 */

static	void	st_func_save()
{
	st_docusave();
	return;
}

/* -----------------------------------------------------------------------------
 * Rename and save the current document.
 */

static	void	st_func_saveas()
{
	if (!st_docusaveas())
		return;

	st_docusave();
	return;
}

/* -----------------------------------------------------------------------------
 * Save the given document. If the document is newly created or imported then
 * it queries the user to get a new name and saves it in the new name.
 */

static	int	st_docusave()
{
register CDROM	*docu = gl_actdocu;
register FILE	*fp;
register int	ret;

	if (!(docu->Settings & GL_DOCUNOTSAVED))
		return(TRUE);

	if ((docu->Settings & GL_DOCUNEW) && !st_docusaveas())
		return(FALSE);

	st_mouse_busy();
	fp = fopen(docu->PathName, "wb");
	if (fp == NULL)
	{
		st_error_message(E_OPENF);
		return(FALSE);
	}

	ret = cd_save(docu, fp);
	fclose(fp);
	st_mint_send_winddraw(docu->PathName);

	if (!ret)
	{
		st_error_message(E_WRITF);
		return(FALSE);
	}

	docu->Settings &= ~(GL_DOCUNEW | GL_DOCUNOTSAVED);
	st_setwindtitle();
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 */

static	int	st_docusaveas()
{
register CDROM	*docu = gl_actdocu;
unsigned char	filename[ MAX_PATHNAME ];

	strcpy(filename, ((docu->Settings & GL_DOCUNEW) ? "" : st_str_filename(docu->PathName)));
	if (!st_get_filename(2, AWRITE, filename, gl_extension, FSEL_SAVE_CDROM))
		return(FALSE);

	st_mouse_busy();
	strcpy(docu->PathName, filename);
	docu->Settings &= ~GL_DOCUNEW;
	docu->Settings |= GL_DOCUNOTSAVED;
	st_setwindtitle();
	return(TRUE);
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * Creates and opens the CD ROM Emulator window
 */

static	int	st_create_window()
{
register int	handle;

	handle = wind_create(gl_windoptions, gl_fullbox.g_x, gl_fullbox.g_y, gl_fullbox.g_w, gl_fullbox.g_h);
	if (handle < 1)
	{
		return(FALSE);
	}

	graf_growbox(1,1,1,1, gl_windrect.g_x, gl_windrect.g_y, gl_windrect.g_w, gl_windrect.g_h);
	if (!wind_open(handle, gl_windrect.g_x, gl_windrect.g_y, gl_windrect.g_w, gl_windrect.g_h))
	{
		wind_delete(handle);
		return(FALSE);
	}

	gl_acthandle = handle;
	st_setwindtitle();
	cd_set_windareas();
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 * Closes and destroys the CD ROM Emulator window
 */

static	void	st_destroy_window()
{
register int	handle = gl_acthandle;
GRECT	box;

	wind_get(handle, WF_FULLXYWH, &box.g_x, &box.g_y, &box.g_w, &box.g_h);
	wind_close(handle);
	wind_delete(handle);
	graf_shrinkbox(1,1,1,1, box.g_x, box.g_y, box.g_w, box.g_h);
	return;
}

/* -----------------------------------------------------------------------------
 * Sets the window title for the given document
 */

static	void	st_setwindtitle()
{
register CDROM	*docu = gl_actdocu;

	sprintf(gl_windtitle, "%s %s", docu->PathName, st_freestr((docu->Settings & GL_DOCUNOTSAVED) ? FR_NOTSAVED : FR_SAVED));
	wind_set(gl_acthandle, WF_NAME, gl_windtitle);
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * The document has been updated.
 */

void	st_set_savedocu()
{
register CDROM	*docu = gl_actdocu;

	if (docu->Settings & GL_DOCUNOTSAVED)
		return;

	docu->Settings |= GL_DOCUNOTSAVED;
	st_setwindtitle();
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

void	st_force_redraw(winhandle, drawrect)
register int	winhandle;
register GRECT	*drawrect;
{
GRECT	b;

	wind_get(winhandle, WF_FIRSTXYWH, &b.g_x, &b.g_y, &b.g_w, &b.g_h);
	while (b.g_w && b.g_h)
	{
		if (rc_intersect(drawrect, &b))
		{
			form_dial(FMD_START, 1,1,1,1, b.g_x, b.g_y, b.g_w, b.g_h);
			form_dial(FMD_FINISH, 1,1,1,1, b.g_x, b.g_y, b.g_w, b.g_h);
		}
		wind_get(winhandle, WF_NEXTXYWH, &b.g_x, &b.g_y, &b.g_w, &b.g_h);
	}
}

/* -----------------------------------------------------------------------------
 * Called for inactive window scrolling.
 */

void	st_display_window(winhandle, drawrect)
int	winhandle;
GRECT	drawrect;
{
int	buffer[8];

	buffer[3] = winhandle;
	buffer[4] = drawrect.g_x;
	buffer[5] = drawrect.g_y;
	buffer[6] = drawrect.g_w;
	buffer[7] = drawrect.g_h;

	st_wind_redraw(buffer);
	return;
}

/* -----------------------------------------------------------------------------
 */

void	st_redraw_window()
{
register int	winhandle = gl_acthandle;
GRECT	b;

	wind_get(winhandle, WF_FIRSTXYWH, &b.g_x, &b.g_y, &b.g_w, &b.g_h);
	while (b.g_w && b.g_h)
	{
		if (!st_mint_dispicon(&b))
		{
			cd_redraw(&b);
		}
		wind_get(winhandle, WF_NEXTXYWH, &b.g_x, &b.g_y, &b.g_w, &b.g_h);
	}
	return;
}
	
/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * sets/resets the common menu parameters
 */

void	st_set_menu()
{
static	int	gl_iconfd_disblist[] =
	{
	CD_NEW   , CD_OPEN  , CD_SAVE  , CD_SAVAS ,
	CD_FILST , CD_LOGFL , CD_EMULT ,
	CD_UNDO  , CD_CUT   , CD_COPY  , CD_PASTE , CD_DELET ,
	CD_INSSS , CD_INSTR , CD_INSFL ,
	CD_SELAL , CD_COMNT , CD_EDFIL ,
	CD_GOSES , CD_GOTRK , CD_FIND  , CD_FINDN ,
	CD_ROMNM , CD_PREF  , CD_PRELD , CD_LTNCY ,
	-1
	};

static	int	gl_inactv_disblist[] =
	{
	CD_SAVE  , CD_SAVAS ,
	CD_FILST , CD_LOGFL , CD_EMULT ,
	CD_UNDO  , CD_CUT   , CD_COPY  , CD_PASTE , CD_DELET ,
	CD_INSSS , CD_INSTR , CD_INSFL ,
	CD_SELAL , CD_COMNT , CD_EDFIL ,
	CD_GOSES , CD_GOTRK , CD_FIND  , CD_FINDN ,
	CD_ROMNM , CD_PREF  , CD_PRELD , CD_LTNCY ,
	-1
	};
static	int	gl_inactv_enablist[] =
	{
	CD_NEW   , CD_OPEN  ,
	-1
	};

static	int	gl_allitm_enablist[] =
	{
	CD_NEW   , CD_OPEN  , CD_SAVE  , CD_SAVAS ,
	CD_FILST , CD_LOGFL , CD_EMULT ,
	CD_UNDO  , CD_CUT   , CD_COPY  , CD_PASTE , CD_DELET ,
	CD_INSSS , CD_INSTR , CD_INSFL ,
	CD_SELAL , CD_COMNT , CD_EDFIL ,
	CD_GOSES , CD_GOTRK , CD_FIND  , CD_FINDN ,
	CD_ROMNM , CD_PREF  , CD_PRELD , CD_LTNCY ,
	-1
	};

register OBJECT	*menu = gl_actmenu;
register int	i;

	menu_ienable(menu, CD_QUIT, 1);

	if (gl_isiconified)
	{
		for (i = 0 ; gl_iconfd_disblist[i] != -1 ; i++)
		{
			menu_ienable(menu, gl_iconfd_disblist[i], 0);
		}
		return;
	}

	if (gl_ispanel)
	{
		for (i = 0 ; gl_inactv_disblist[i] != -1 ; i++)
		{
			menu_ienable(menu, gl_inactv_disblist[i], 0);
		}
		for (i = 0 ; gl_inactv_enablist[i] != -1 ; i++)
		{
			menu_ienable(menu, gl_inactv_enablist[i], 1);
		}
		return;
	}

	for (i = 0 ; gl_allitm_enablist[i] != -1 ; i++)
	{
		menu_ienable(menu, gl_allitm_enablist[i], 1);
	}

	cd_setmenu_rest();
	cd_setmenu_empty();
	cd_setmenu_insert();
	cd_setmenu_select();
	cd_setmenu_undo();
	return;
}

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	int	st_mint_dispicon(drawrect)
GRECT	*drawrect;
{
register OBJECT	*Tree;
register void	*temp;
register int	w, h;
GRECT	a, b;

	if (!gl_isiconified)
		return(FALSE);

	wind_get(gl_acthandle, WF_WORKXYWH, &b.g_x, &b.g_y, &b.g_w, &b.g_h);
	if (!st_intersect(&b, drawrect, &a))
		return(TRUE);

	Tree = st_objc_get_tree(CD_ICNFD);

	Tree[ CDICNBOX ].ob_x = b.g_x;
	Tree[ CDICNBOX ].ob_y = b.g_y;
	Tree[ CDICNBOX ].ob_width = b.g_w;
	Tree[ CDICNBOX ].ob_height = b.g_h;

	temp = Tree[ CDROMICN ].ob_spec;
	w = ((BITBLK *)temp)->bi_wb;
	h = ((BITBLK *)temp)->bi_hl;

	Tree[ CDROMICN ].ob_x = (b.g_w - w) / 2;
	Tree[ CDROMICN ].ob_y = (b.g_h - h) / 2;
	Tree[ CDROMICN ].ob_width = w;
	Tree[ CDROMICN ].ob_height = h;

	objc_draw(Tree, ROOT, MAX_DEPTH, a.g_x, a.g_y, a.g_w, a.g_h);
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 */

void	st_mint_wind_get(box)
register GRECT	*box;
{
register int	type;

	type = (gl_isiconified ? WF_UNICONIFY : WF_CURRXYWH);
	wind_get(gl_acthandle, type, &(box->g_x), &(box->g_y), &(box->g_w), &(box->g_h));
	return;
}

/* -----------------------------------------------------------------------------
 */

void	st_mint_send_winddraw(pathname)
unsigned char	pathname[];
{
unsigned char	app_name[20];
int	app_id, app_type;
int	message[8];
	
	if (!gl_mint_installed || (gl_mint_aesversion < VERSION_AES_MULTITOS400))
		return;

	wind_update(END_UPDATE);
	app_id = appl_search(2, app_name, &app_type, &app_id);
	if (app_id == 0)
		return;

	message[0] = SH_WDRAW;
	message[1] = gl_appid;
	message[2] = 0;
	message[3] = (int)(pathname[0] - 'A');
	message[4] = 0;
	message[5] = 0;
	message[6] = 0;
	message[7] = 0;

	appl_write(app_id, 16, message);

	wind_update(BEG_UPDATE);
	return;
}

/******************************************************************************/
/* End of ST_GLOB.C							      */
/******************************************************************************/
