/******************************************************************************/
/* ST_RSRC.C								      */
/******************************************************************************/

/* -----------------------------------------------------------------------------
 * - The function in this file will load the resource file for Works.
 *   If the resource file for the country settings is not found, then try
 *   to load the default resource file.
 *
 * Stuff added for additional resource to be loaded from the resource for
 * free strings and form_alerts.
 */

#include "st_def.h"
#include "st_aes.h"
#include "st_tos.h"

#include "cdstrusa.h"

/* -------------------------------------------------------------------------- */

extern	int	gl_tos_country;

extern	unsigned char	gl_pathname[];
extern	unsigned char	gl_sysname[];

/* -------------------------------------------------------------------------- */

/* Array Resource filenames indexed according to */
/* TOS country codes DEFAULT is USA always */

#define	MAX_RSRCFILE	MAX_TOS_LANGUAGE

static	unsigned char	*rsrc_filenames[][3] =
	{
	"CDMONUSA.RSC",	"CDCOLUSA.RSC",	"CDSTRUSA.RSC",	/* English */
	"CDMONGRM.RSC",	"CDCOLGRM.RSC",	"CDSTRGRM.RSC",	/* German */
	"CDMONFRN.RSC",	"CDCOLFRN.RSC",	"CDSTRFRN.RSC",	/* French */
	"CDMONUSA.RSC",	"CDCOLUSA.RSC",	"CDSTRUSA.RSC",	/* British */
	"CDMONSPN.RSC",	"CDCOLSPN.RSC",	"CDSTRSPN.RSC",	/* Spanish */
	"CDMONITA.RSC",	"CDCOLITA.RSC",	"CDSTRITA.RSC",	/* Italian */
	"CDMONSWE.RSC",	"CDCOLSWE.RSC",	"CDSTRSWE.RSC",	/* Swedish */
	"CDMONFRN.RSC",	"CDCOLFRN.RSC",	"CDSTRFRN.RSC",	/* Swiss French */
	"CDMONGRM.RSC",	"CDCOLGRM.RSC",	"CDSTRGRM.RSC",	/* Swiss German */
	"CDMONTRK.RSC",	"CDCOLTRK.RSC",	"CDSTRTRK.RSC",	/* Turkish */
	"CDMONFIN.RSC",	"CDCOLFIN.RSC",	"CDSTRFIN.RSC",	/* Finish */
	"CDMONNOR.RSC",	"CDCOLNOR.RSC",	"CDSTRNOR.RSC",	/* Norwegian */
	"CDMONDAN.RSC",	"CDCOLDAN.RSC",	"CDSTRDAN.RSC",	/* Danish */
	"CDMONARA.RSC",	"CDCOLARA.RSC",	"CDSTRARA.RSC",	/* Arabic */
	"CDMONDUT.RSC",	"CDCOLDUT.RSC",	"CDSTRDUT.RSC",	/* Dutch */
	"CDMONCHK.RSC", "CDCOLCHK.RSC", "CDSTRCHK.RSC",	/* Czechoslovakia */
	"CDMONHNG.RSC", "CDCOLHNG.RSC", "CDSTRHNG.RSC"	/* Hungery */
	};

/* -------------------------------------------------------------------------- */

static	int	st_load_given_resource( unsigned char *, int, int );
static	int	st_load_given_freestrs( int, unsigned char **, int *, int );

/******************************************************************************/
/******************************************************************************/

/* -----------------------------------------------------------------------------
 */

static	int	st_load_given_resource(pathname, i, which)
unsigned char	pathname[];
int	i, which;
{
unsigned char	rsrcname[ MAX_PATHNAME ];

	strcpy(rsrcname, pathname);
	strcat(rsrcname, gl_sysname);
	strcat(rsrcname, rsrc_filenames[i][ which ]);

	return(rsrc_load(rsrcname) ? TRUE : FALSE);
}

/* -----------------------------------------------------------------------------
 */

int	st_load_resource(which)
register int	which;
{
register int	i;

	if (st_load_given_resource(gl_pathname, gl_tos_country, which) ||
	    st_load_given_resource("\\", gl_tos_country, which) ||
	    st_load_given_resource(gl_pathname, 0, which) ||
	    st_load_given_resource("\\", 0, which))
		return(TRUE);

	for (i = 0 ; i < MAX_RSRCFILE ; i++)
	{
		if (st_load_given_resource(gl_pathname, i, which) ||
		    st_load_given_resource("\\", i, which))
			return(TRUE);
	}

	return(FALSE);
}

/******************************************************************************/
/******************************************************************************/

/* -------------------------------------------------------------------------- */

#define	RSRC_ALERT_MAX_INDEX_0	21
#define	RSRC_ALERT_MAX_INDEX_1	1
#define	RSRC_ALERT_MAX_INDEX_2	0
#define	RSRC_ALERT_MAX_INDEX_3	2

#define	RSRC_MAX_FREE_STRS	122

/* -------------------------------------------------------------------------- */

static	int	rsrc_alert_index_0[] =
	{
	_MALL, _OPENF, _READF, _WRITF,
	_SCRPINI, _FILEXST, _WINDLIM,
	_FSI, _INVFNAM, _ACCESSF,
	_SCRPFTL, _INVPARM,
	_CDROMNM,
	_UNDOERR,
	_NOFNDST,
	_OPENLOG, _WRITLOG,

	_NOI2SLD, _SNDLCKD, _I2SNOMM, _I2SLDER,
	-1
	};

static	int	rsrc_alert_index_1[] =
	{
	_SAVEDOC,
	-1
	};

static	int	rsrc_alert_index_2[] =
	{
	-1
	};

static	int	rsrc_alert_index_3[] =
	{
	_NOSESSN,
	_NOTRACK,
	-1
	};

static	int	freestr_index[] =
	{
	/* ---- */

	FRABOUT0, FRABOUT1,
	FRABOUT2, FRABOUT3,
	FRABOUT4,

	/* ---- */

	FSELSV01, FSELLD01,
	FSELSV02, FSELLD02,
	FSELLOGF,
	FSELFILE,

	/* ---- */

	FRUNTTLD,
	FRYESSVD, FRNOTSVD,

	/* ---- */

	FRINFORM,
	FRSESSNO, FRTRAKNO,
	FRLASTLN,
	FRFILNAM, FRABSQTM, FRRELQTM,

	/* ---- */

	FRSIZE, FRSTART, FREND, FRCOMMNT,

	/* ---- */

	FRINIEMU, FRCHKFIL, FRCHECKS, FRCHECKT,
	FRFNOTEX, FRFYESEX, FRQTIMSB,
	FRI2SBUS, FRDSABUS, FRPARAMS,
	FRQITEMU,

	/* ---- */

	FRRSRVED, FRUNDEFN, FRUSRDEF,

	/* ---- */

	FRPLAYTR, FRSTOPCD, FRREDTOC,
	FRPAUSCD, FRUNPAUS, FRGETLEN,
	FROPNTRA, FRCLSTRA, FRGETCMP,

	FRABSMIN, FRABSSEC, FRABSFRM,
	FRREDLNG, FRSETMOD, FRGETERR,
	FRCLRERR, FRSPINUP,

	FRPLABSM, FRPLABSS, FRPLABSF,
	FRPLABEM, FRPLABES, FRPLABEF,
	FRRELSAB,

	FRGTSTAT, FRSETVOL, FRSESINF,
	FRDACMOD,

	FRSRVMOD, FRSLGMOD, FRFOCUSL,
	FRTRNTAB, FRRADIAL,

	/* ---- */

	FRFOUND , FRSTOPED, FRDISCST, FRERRVAL,

	FRLENLSB, FRLENMSB,
	FRTRAMOV, FRTRAOPN, FRTRACLS,

	FRACTTTL, FRACTIND, FRACTMIN,
	FRACTSEC, FRABSTMM, FRABSTMS,
	FRABSTMF, FRMODEST,

	FRTOCMNT, FRTOCMXT, FRTOCSTM,
	FRTOCSTS, FRTOCSTF,

	FRABRLSD, FRVOLLVL, FRINFSES,

	FRLTOCTR, FRLTOCAT, FRLTOCSM,
	FRLTOCSS, FRLTOCSF,

	FRMODDAC, FRSRVVER,

	/* ---- */

	FREUNDEF, FRENOERR,
	FREFOCUS, FRESBCOD, FRERDTOC,
	FRERDIAL, FRESLDGE, FRETRNTB,
	FRESRCHO, FRESRCHB,
	FRESRCHI, FRESRCHT,
	FREILCOM, FREILVAL, FREILTIM,
	FRECOMME, FRETRAYE, FREHFDET,

	/* ---- */

	-1
	};

/* -------------------------------------------------------------------------- */

static	int	error_code_0[] =
	{
	E_MALL, E_OPENF, E_READF, E_WRITF, 
	E_SCRAPINIT, E_FILEXIST, E_WINDLIM,
	E_FSI,  E_INVALIDFNAME, E_ACCESSF,
	E_FATALSCRAP, E_INVALIDPARM, 
	E_CDROMTITLE,
	E_UNDOERR,
	E_STRNOTFOUND,
	E_LOG_OPEN, E_LOG_WRITE,

	E_I2S_NOLODFILE, E_I2S_SOUNDLOCKED, E_I2S_NOMEMORY, E_I2S_LOADERROR,
	-10000
	};

static	int	error_code_1[] =
	{
	E_SAVEDOCU, 
	-10000
	};

static	int	error_code_2[] =
	{
	-10000
	};

static	int	error_code_3[] =
	{
	E_NOSESSION,
	E_NOTRACK,
	-10000
	};

static	int	freestr_code[] =
	{
	/* ---- */

	DIAL_INFO_AUTHOR, DIAL_INFO_LOAD_RESOURCE,
	DIAL_INFO_LOAD_FONTS, DIAL_INFO_INITIALISING,
	DIAL_INFO_OKBUTTON,

	/* ---- */

	FSEL_SAVE_CDROM, FSEL_LOAD_CDROM,
	FSEL_SAVE_TOC, FSEL_LOAD_TOC,
	FSEL_LOGFILENAME,
	FSEL_ITEMFILENAME,

	/* ---- */

	FR_UNTITLED,
	FR_SAVED, FR_NOTSAVED,

	/* ---- */

	FR_INFORMATION,
	FR_SESSION, FR_TRACK,
	FR_LASTLINE,
	FR_FILENAME, FR_ABSQTIMESUBCODE, FR_RELQTIMESUBCODE,

	/* ---- */

	FR_SIZE, FR_STARTTIME, FR_ENDTIME, FR_COMMENT,

	/* ---- */

	FR_INIT_EMULATOR, FR_CHECK_CDROM, FR_CHECK_SESSION, FR_CHECK_TRACK,
	FR_CHECK_FILES_NOT, FR_CHECK_FILES_YES, FR_UPDATE_QTIME,
	FR_INIT_I2SBUS, FR_INIT_DSABUS, FR_INIT_PARAMS,
	FR_QUIT_EMULATION,

	/* ---- */

	FR_RESERVED, FR_UNDEFINED, FR_USERDEFINED,

	/* ---- */

	FR_PLAYTITLE, FR_STOP, FR_READTOC,
	FR_PAUSE, FR_UNPAUSE, FR_GETTITLELEN,
	FR_OPENTRAY, FR_CLOSETRAY, FR_GETCOMPTIME,

	FR_GOTOABSMIN, FR_GOTOABSSEC, FR_GOTOABSFRM,
	FR_READLONGTOC, FR_SETMODE, FR_GETERROR,
	FR_CLEARERROR, FR_SPINUP,

	FR_PLAB_STRTMIN, FR_PLAB_STRTSEC, FR_PLAB_STRTFRM,
	FR_PLAB_STOPMIN, FR_PLAB_STOPSEC, FR_PLAB_STOPFRM,
	FR_RELEASEAB,

	FR_GETDISCSTAT, FR_SETVOLUME, FR_SESSIONINFO,
	FR_SETDACMODE,

	FR_SERVICE, FR_SLEDGE, FR_FOCUS	,
	FR_TURNTABLE, FR_RADIAL,

	/* ---- */

	FR_FOUND, FR_STOPPED, FR_DISCSTATUS, FR_ERRORVALUES,

	FR_LENTITLELSB, FR_LENTITLEMSB,
	FR_TRAYMOVING, FR_TRAYOPEN, FR_TRAYCLOSED,

	FR_ACTUALTITLE, FR_ACTUALINDEX, FR_ACTUALMINUTES,
	FR_ACTUALSECONDS, FR_ABSTIMEMINS, FR_ABSTIMESECS,
	FR_ABSTIMEFRMS, FR_MODESTATUS,

	FR_TOCVAL_MINTRK, FR_TOCVAL_MAXTRK, FR_TOCVAL_STARTMINS,
	FR_TOCVAL_STARTSECS, FR_TOCVAL_STARTFRMS,

	FR_ABRELEASED, FR_VOLUMELEVEL, FR_MAXSESSIONS,

	FR_LONGTOC_TRACK, FR_LONGTOC_ATTR, FR_LONGTOC_STARTMINS,
	FR_LONGTOC_STARTSECS, FR_LONGTOC_STARTFRMS,

	FR_DACMODE, FR_SERVOVERSION,

	/* ---- */

	FR_E_UNDEFINED, FR_E_NOERROR,
	FR_E_FOCUS, FR_E_SUBCODE, FR_E_READTOC,
	FR_E_RADIAL, FR_E_FATALSLEDGE, FR_E_TURNTABLE,
	FR_E_SEARCHTIMEOUT, FR_E_SEARCHBINARY,
	FR_E_SEARCHINDEX, FR_E_SEARCHTIME,
	FR_E_ILLEGALCOMMAND, FR_E_ILLEGALVALUE, FR_E_ILLEGALTIME,
	FR_E_COMMUNICATION, FR_E_TRAY, FR_E_HFDETECTOR,

	/* ---- */

	-10000
	};

/* -------------------------------------------------------------------------- */

static	unsigned char	*rsrc_alert_str_0[ RSRC_ALERT_MAX_INDEX_0 + 1];
static	unsigned char	*rsrc_alert_str_1[ RSRC_ALERT_MAX_INDEX_1 + 1];
static	unsigned char	*rsrc_alert_str_2[ RSRC_ALERT_MAX_INDEX_2 + 1];
static	unsigned char	*rsrc_alert_str_3[ RSRC_ALERT_MAX_INDEX_3 + 1];

static	unsigned char	*freestr_ptr[ RSRC_MAX_FREE_STRS + 1 ];

/* -----------------------------------------------------------------------------
 */

static	int	st_load_given_freestrs(which, s, indexlist, maxno)
register int	which;
register unsigned char	*s[];
register int	indexlist[];
register int	maxno;
{
register unsigned char	*cp;
register int	i, len;
OBJECT	*Tree;

	Tree = (which ? NULL : st_objc_get_tree(FREESTRS));
	for (i = 0 ; i < maxno ; i++)
	{
		cp = (which ? st_objc_get_string(indexlist[i]) : Tree[ indexlist[i] ].ob_spec);
		len = (int)strlen(cp);

		s[i] = malloc((size_t)(len+1));
		if (s[i] == NULL)
			return(FALSE);

		strcpy(s[i], cp);
	}
	s[i] = "";
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 */

int	st_load_freestrs()
{
	if (!st_load_resource(2))
	{
		st_error_load_resource();
		return(FALSE);
	}

	if (!st_load_given_freestrs(TRUE, rsrc_alert_str_0, rsrc_alert_index_0, RSRC_ALERT_MAX_INDEX_0) ||
	    !st_load_given_freestrs(TRUE, rsrc_alert_str_1, rsrc_alert_index_1, RSRC_ALERT_MAX_INDEX_1) ||
	    !st_load_given_freestrs(TRUE, rsrc_alert_str_2, rsrc_alert_index_2, RSRC_ALERT_MAX_INDEX_2) ||
	    !st_load_given_freestrs(TRUE, rsrc_alert_str_3, rsrc_alert_index_3, RSRC_ALERT_MAX_INDEX_3) ||
	    !st_load_given_freestrs(FALSE, freestr_ptr, freestr_index, RSRC_MAX_FREE_STRS))
	{
		form_error(8);
		return(FALSE);
	}

	rsrc_free();
	return(TRUE);
}

/* -----------------------------------------------------------------------------
 * Displays the given error message
 */

int	st_error_message(errflag)
register int	errflag;
{
register int	i;

	for (i = 0 ;; i++)
	{
		if (error_code_0[i] == -10000)
			break;

		if (error_code_0[i] == errflag)
			return(form_alert(1, rsrc_alert_str_0[i]));
	}
	return(1);
}

/* -----------------------------------------------------------------------------
 * Routine to DISABLE/ENABLE/RESTORE menu status whenever required.
 */

int	st_error_message_one(errflag, longval)
register int	errflag;
register long	longval;
{
register int	i;
unsigned char	s[ MAX_STRLEN ];

	for (i = 0 ;; i++)
	{
		if (error_code_1[i] == -10000)
			break;

		if (error_code_1[i] == errflag)
		{
			sprintf(s, rsrc_alert_str_1[i], longval);
			return(form_alert(1, s));
		}
	}
	return(1);
}

/* -----------------------------------------------------------------------------
 */

int	st_error_message_two(errflag, longval1, longval2)
register int	errflag;
register long	longval1;
register long	longval2;
{
register int	i;
unsigned char	s[ MAX_STRLEN ];

	for (i = 0 ;; i++)
	{
		if (error_code_2[i] == -10000)
			break;

		if (error_code_2[i] == errflag)
		{
			sprintf(s, rsrc_alert_str_2[i], longval1, longval2);
			return(form_alert(1, s));
		}
	}
	return(1);
}

/* -----------------------------------------------------------------------------
 */

int	st_error_message_three(errflag, longval1, longval2, longval3)
register int	errflag;
register long	longval1;
register long	longval2;
register long	longval3;
{
register int	i;
unsigned char	s[ MAX_STRLEN ];

	for (i = 0 ;; i++)
	{
		if (error_code_3[i] == -10000)
			break;

		if (error_code_3[i] == errflag)
		{
			sprintf(s, rsrc_alert_str_3[i], longval1, longval2, longval3);
			return(form_alert(1, s));
		}
	}
	return(1);
}

/* -----------------------------------------------------------------------------
 */

unsigned char	*st_freestr(code)
register int	code;
{
register int	i;

	for (i = 0 ; (i < RSRC_MAX_FREE_STRS) && (freestr_code[i] != code) ; i++);

	return(freestr_ptr[i]);
}

/******************************************************************************/
/* End of ST_RSRC.C							      */
/******************************************************************************/
